# RELation-aware Adaptive Cross-attention (RELAX)
Official repository for
Image-Text Alignment using Adaptive Cross-attention with Transformer Encoder for Scene Graphs which is accepted to
British Machine Vision Conference (BMVC) 2021.


To see the results of the paper, run the code.
```
python3 ensemble_by_pkls.py sims_f30k/sims_f30k_RELAX_i2t.pkl sims_f30k/sims_f30k_RELAX_t2i.pkl
```
(We saved the `sims_f30k...` files in JSON on our training codes, but we changed them to .pkl files to reduce the size.)

# Training
We also provide the script files for the training of Flickr30k[5].
## Preparing Data
Download Flickr30k images: https://www.kaggle.com/hsankesara/flickr-image-dataset

Extract .zip to your `$RELAX_PATH`, (`export RELAX_PATH=$PWD`)

```
cd codes
unzip data.zip
python pkl_to_json.py
```

## BUA setup 

To run it, one needs to make Bottom-up attention(BUA)[1] features visually grounded by NeuralMotif[6].
The scene graphs are provided in folder './data'.

```
git clone https://github.com/peteanderson80/bottom-up-attention.git
export BUA_PATH=$PWD/bottom-up-attention
```
Build Caffe following installation guide in BUA repository.
Also, you should prepare a pretrained Caffe model. 
(https://storage.googleapis.com/up-down-attention/resnet101_faster_rcnn_final.caffemodel)

```
cp ${RELAX_PATH}/tools/*.ipynb ${BUA_PATH}/tools/
cp ${RELAX_PATH}/lib/fast_rcnn/test.py ${BUA_PATH}/lib/fast_rcnn/
```

## Generate feature

Now, you can generate features using copied .ipynb files.

## Run

After preparing the data, run below.

```
sh run_t2i.sh
sh run_i2t.sh
```

Note that it is needed high memory GPUs such as v100 for batch_size=128.
In Titan-XP, it is needed to set small batch_size, such as 20.

## Citation

If you found this code useful, please cite our paper.

```
@article{song2021relax,
  title={Image-Text Alignment using Adaptive Cross-attention with Transformer Encoder for Scene Graphs},
  author={Song, Juyong and Choi, Sunghyun},
  journal={31st British Machine Vision Conference},
  year={2021}
}
```

Our codes refer to [2],[3], and [4].

[1] Peter Anderson, Xiaodong He, Chris Buehler, Damien Teney, Mark Johnson, Stephen Gould, and Lei Zhang. Bottom-up and top-down attention for image captioning and visual question answering. In CVPR, 2018.

[2] Kuang-Huei Lee, Xi Chen, Gang Hua, Houdong Hu, and Xiaodong He. Stacked cross attention for image-text matching. In ECCV, 2018

[3] Sijin Wang, Ruiping Wang, Ziwei Yao, Shiguang Shan, and Xilin Chen. Cross-modal scene graph matching for relationship-aware image-text retrieval. In WACV, 2020.

[4] Jonatas Wehrmann, Camila Kolling, and Rodrigo C Barros. Adaptive cross-modal embeddings for image-text alignment. In AAAI, 2020

[5] Peter Young, Alice Lai, Micah Hodosh, and Julia Hockenmaier.  From image descriptions to visual denotations: New similarity metrics for semantic inference over event descriptions. TACL, 2014.

[6] Rowan Zellers, Mark Yatskar, Sam Thomson, and Yejin Choi. Neural motifs: Scene graph parsing with global context. In CVPR, 2018.

