import numpy as np
import json
import os
import sys
import pickle

if len(sys.argv) < 3:
    print('put the file list to make an ensemble')
    print('python3 ensemble_by_sims.py I2T_model T2I_model fold5(True/False)')
    exit(1)

i2t_result_file = sys.argv[1]
t2i_result_file = sys.argv[2]
if len(sys.argv) == 4:
    fold5 = bool(sys.argv[3])
else:
    fold5 = False

print(i2t_result_file)
print(t2i_result_file)

with open(i2t_result_file, 'rb') as f:
    i2t_results = pickle.load(f)
with open(t2i_result_file, 'rb') as f:
    t2i_results = pickle.load(f)

sims = []

if fold5:
    i2t_sims = np.array(i2t_results['sims_fold5'])
    t2i_sims = np.array(t2i_results['sims_fold5'])
else:
    i2t_sims = np.array(i2t_results)
    t2i_sims = np.array(t2i_results)

sims.append(i2t_sims)
sims.append(t2i_sims)
sims = np.array(sims)

sims = np.mean(sims, axis=0)
print(sims.shape)

from evaluation import i2t, t2i

if fold5:
    r_i2t = []
    ri_i2t = []
    r_t2i = []
    ri_t2i = []
    for i in range(5):
        print("I2T")
        r, rt, i2t_scores = i2t(None, None, None, i2t_sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(None, None, None, i2t_sims[i], return_ranks=True)
        r_i2t.append(r)
        ri_i2t.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')

        print("T2I")
        r, rt, i2t_scores = i2t(None, None, None, t2i_sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(None, None, None, t2i_sims[i], return_ranks=True)
        r_t2i.append(r)
        ri_t2i.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')

    r_sum = []
    ri_sum = []
    for i in range(5):
        r, rt, i2t_scores = i2t(None, None, None, sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(None, None, None, sims[i], return_ranks=True)
        r_sum.append(r)
        ri_sum.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')

    r_ens = np.array([list(r_s) for r_s in r_sum]).mean(0)
    ri_ens = np.array([list(ri_s) for ri_s in ri_sum]).mean(0)
    print(r_ens[:3], ri_ens[:3])
    print(r_ens[:3].sum(), ri_ens[:3].sum(), r_ens[:3].sum()+ri_ens[:3].sum())

else:
    r, rt, i2t_scores = i2t(None, None, None, i2t_sims, return_ranks=True)
    ri, rti, t2i_scores = t2i(None, None, None, i2t_sims, return_ranks=True)
    ar = (r[0] + r[1] + r[2]) / 3
    ari = (ri[0] + ri[1] + ri[2]) / 3
    rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
    print("I2T model")
    print("rsum: %.1f" % rsum)
    print("Average i2t Recall: %.1f" % ar)
    print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
    print("Average t2i Recall: %.1f" % ari)
    print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
    print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')
   
    
    r, rt, i2t_scores = i2t(None, None, None, t2i_sims, return_ranks=True)
    ri, rti, t2i_scores = t2i(None, None, None, t2i_sims, return_ranks=True)
    ar = (r[0] + r[1] + r[2]) / 3
    ari = (ri[0] + ri[1] + ri[2]) / 3
    rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
    print("T2I model")
    print("rsum: %.1f" % rsum)
    print("Average i2t Recall: %.1f" % ar)
    print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
    print("Average t2i Recall: %.1f" % ari)
    print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
    print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')
   
    
    r, rt, i2t_scores = i2t(None, None, None, sims, return_ranks=True)
    ri, rti, t2i_scores = t2i(None, None, None, sims, return_ranks=True)
    ar = (r[0] + r[1] + r[2]) / 3
    ari = (ri[0] + ri[1] + ri[2]) / 3
    rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
    print("rsum: %.1f" % rsum)
    print("Average i2t Recall: %.1f" % ar)
    print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
    print("Average t2i Recall: %.1f" % ari)
    print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
    print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')
   

