import numpy as np
import json
import os
from evaluation import i2t, t2i

import argparse


parser = argparse.ArgumentParser()

parser.add_argument('--outpath', default='./BMVC_f30k/recalls')
parser.add_argument('--t2i_model')
parser.add_argument('--i2t_model')
parser.add_argument('--fold5', action='store_true')

args = parser.parse_args()

if not os.path.exists(args.t2i_model):
    print('t2i_model does not exist')
    exit(1)

if not os.path.exists(args.i2t_model):
    print('i2t_model does not exist')
    exit(1)


t2i_result_file = args.t2i_model
i2t_result_file = args.i2t_model


with open(t2i_result_file) as f:
    t2i_results = json.load(f)
with open(i2t_result_file) as f:
    i2t_results = json.load(f)


sims = []
t2i_sims = np.array(t2i_results['sims'])
i2t_sims = np.array(i2t_results['sims'])
sims.append(t2i_sims)
sims.append(i2t_sims)
sims = np.array(sims)

sims = np.mean(sims, axis=0)

print(sims.shape)
fold5=args.fold5
_ = []
if fold5:
    r_i2t = []
    ri_i2t = []
    r_t2i = []
    ri_t2i = []
    for i in range(5):
        print("I2T")
        r, rt, i2t_scores = i2t(_, _, _, i2t_sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(_, _, _, i2t_sims[i], return_ranks=True)
        r_i2t.append(r)
        ri_i2t.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ar} {ri[0]} {ri[1]} {ri[2]} {ari}')

        print("T2I")
        r, rt, i2t_scores = i2t(_, _, _, t2i_sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(_, _, _, t2i_sims[i], return_ranks=True)
        r_t2i.append(r)
        ri_t2i.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')


    r_i2t_mean = np.array([list(r_s) for r_s in r_i2t]).mean(0)
    ri_i2t_mean = np.array([list(ri_s) for ri_s in ri_i2t]).mean(0)
    print(r_i2t_mean[:3], ri_i2t_mean[:3])
    print(r_i2t_mean[:3].sum(), ri_i2t_mean[:3].sum(), r_i2t_mean[:3].sum()+ri_i2t_mean[:3].sum())
    ar = (r_i2t_mean[0] + r_i2t_mean[1] + r_i2t_mean[2]) / 3
    ari = (ri_i2t_mean[0] + ri_i2t_mean[1] + ri_i2t_mean[2]) / 3
    rsum = r_i2t_mean[0] + r_i2t_mean[1] + r_i2t_mean[2] + ri_i2t_mean[0] + ri_i2t_mean[1] + ri_i2t_mean[2]
    i2t_model_name = os.path.basename(args.i2t_model)
    with open(os.path.join(args.outpath, 'fold5_'+i2t_model_name[:-4]+'txt'), 'w') as f:
        f.write(f'{rsum:.2f} {r_i2t_mean[0]:.2f} {r_i2t_mean[1]:.2f} {r_i2t_mean[2]:.2f} {ar:.2f} {ri_i2t_mean[0]:.2f} {ri_i2t_mean[1]:.2f} {ri_i2t_mean[2]:.2f} {ari:.2f}')


    r_t2i_mean = np.array([list(r_s) for r_s in r_t2i]).mean(0)
    ri_t2i_mean = np.array([list(ri_s) for ri_s in ri_t2i]).mean(0)
    print(r_t2i_mean[:3], ri_t2i_mean[:3])
    print(r_t2i_mean[:3].sum(), ri_t2i_mean[:3].sum(), r_t2i_mean[:3].sum()+ri_t2i_mean[:3].sum())
    ar = (r_t2i_mean[0] + r_t2i_mean[1] + r_t2i_mean[2]) / 3
    ari = (ri_t2i_mean[0] + ri_t2i_mean[1] + ri_t2i_mean[2]) / 3
    rsum = r_t2i_mean[0] + r_t2i_mean[1] + r_t2i_mean[2] + ri_t2i_mean[0] + ri_t2i_mean[1] + ri_t2i_mean[2]
    t2i_model_name = os.path.basename(args.t2i_model)
    with open(os.path.join(args.outpath, 'fold5_'+t2i_model_name[:-4]+'txt'), 'w') as f:
        f.write(f'{rsum:.2f} {r_t2i_mean[0]:.2f} {r_t2i_mean[1]:.2f} {r_t2i_mean[2]:.2f} {ar:.2f} {ri_t2i_mean[0]:.2f} {ri_t2i_mean[1]:.2f} {ri_t2i_mean[2]:.2f} {ari:.2f}')


    r_sum = []
    ri_sum = []
    for i in range(5):
        r, rt, i2t_scores = i2t(_, _, _, sims[i], return_ranks=True)
        ri, rti, t2i_scores = t2i(_, _, _, sims[i], return_ranks=True)
        r_sum.append(r)
        ri_sum.append(ri)
        ar = (r[0] + r[1] + r[2]) / 3
        ari = (ri[0] + ri[1] + ri[2]) / 3
        rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
        print("rsum: %.1f" % rsum)
        print("Average i2t Recall: %.1f" % ar)
        print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
        print("Average t2i Recall: %.1f" % ari)
        print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
        print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')

    r_ens = np.array([list(r_s) for r_s in r_sum]).mean(0)
    ri_ens = np.array([list(ri_s) for ri_s in ri_sum]).mean(0)
    print(r_ens[:3], ri_ens[:3])
    print(r_ens[:3].sum(), ri_ens[:3].sum(), r_ens[:3].sum()+ri_ens[:3].sum())
    ar = (r_ens[0] + r_ens[1] + r_ens[2]) / 3
    ari = (ri_ens[0] + ri_ens[1] + ri_ens[2]) / 3
    rsum = r_ens[0] + r_ens[1] + r_ens[2] + ri_ens[0] + ri_ens[1] + ri_ens[2]

    ens_model_name = t2i_model_name[:-10] + '_' + i2t_model_name[:-10]
    with open(os.path.join(args.outpath, 'fold5_ens_' + ens_model_name + '.txt'), 'w') as f:
        f.write(f'{rsum:.2f} {r_ens[0]:.2f} {r_ens[1]:.2f} {r_ens[2]:.2f} {ar:.2f} {ri_ens[0]:.2f} {ri_ens[1]:.2f} {ri_ens[2]:.2f} {ari:.2f}')



_ = []
r, rt, i2t_scores = i2t(_, _, _, i2t_sims, return_ranks=True)
ri, rti, t2i_scores = t2i(_, _, _, i2t_sims, return_ranks=True)
ar = (r[0] + r[1] + r[2]) / 3
ari = (ri[0] + ri[1] + ri[2]) / 3
rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
print('i2t attention model')
print("rsum: %.1f" % rsum)
print("Average i2t Recall: %.1f" % ar)
print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
print("Average t2i Recall: %.1f" % ari)
print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
print(f'{rsum} {r[0]} {r[1]} {r[2]} {ar} {ri[0]} {ri[1]} {ri[2]} {ari}')
i2t_model_name = os.path.basename(args.i2t_model)
with open(os.path.join(args.outpath, i2t_model_name[:-4]+'txt'), 'w') as f:
    f.write(f'{rsum:.2f} {r[0]:.2f} {r[1]:.2f} {r[2]:.2f} {ar:.2f} {ri[0]:.2f} {ri[1]:.2f} {ri[2]:.2f} {ari:.2f}')

r, rt, i2t_scores = i2t(_, _, _, t2i_sims, return_ranks=True)
ri, rti, t2i_scores = t2i(_, _, _, t2i_sims, return_ranks=True)
ar = (r[0] + r[1] + r[2]) / 3
ari = (ri[0] + ri[1] + ri[2]) / 3
rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]
print('t2i attention model')
print("rsum: %.1f" % rsum)
print("Average i2t Recall: %.1f" % ar)
print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
print("Average t2i Recall: %.1f" % ari)
print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')
t2i_model_name = os.path.basename(args.t2i_model)
with open(os.path.join(args.outpath, t2i_model_name[:-4]+'txt'), 'w') as f:
    f.write(f'{rsum:.2f} {r[0]:.2f} {r[1]:.2f} {r[2]:.2f} {ar:.2f} {ri[0]:.2f} {ri[1]:.2f} {ri[2]:.2f} {ari:.2f}')


r, rt, i2t_scores = i2t(_, _, _, sims, return_ranks=True)
ri, rti, t2i_scores = t2i(_, _, _, sims, return_ranks=True)
ar = (r[0] + r[1] + r[2]) / 3
ari = (ri[0] + ri[1] + ri[2]) / 3
rsum = r[0] + r[1] + r[2] + ri[0] + ri[1] + ri[2]

print('Ensemble')
print("rsum: %.1f" % rsum)
print("Average i2t Recall: %.1f" % ar)
print("Image to text: %.1f %.1f %.1f %.1f %.1f" % r)
print("Average t2i Recall: %.1f" % ari)
print("Text to image: %.1f %.1f %.1f %.1f %.1f" % ri)
print(f'{rsum} {r[0]} {r[1]} {r[2]} {ri[0]} {ri[1]} {ri[2]}')
# ens_model_name = t2i_model_name.replace('t2i', 'ens')
# with open(os.path.join(args.outpath, ens_model_name[:-4]+'txt'), 'w') as f:
#     f.write(f'{rsum:.2f} {r[0]:.2f} {r[1]:.2f} {r[2]:.2f} {ar:.2f} {ri[0]:.2f} {ri[1]:.2f} {ri[2]:.2f} {ari:.2f}')

ens_model_name = t2i_model_name[:-10] + '_' + i2t_model_name[:-10]
with open(os.path.join(args.outpath, 'ens_' + ens_model_name + '.txt'), 'w') as f:
    f.write(f'{rsum:.2f} {r[0]:.2f} {r[1]:.2f} {r[2]:.2f} {ar:.2f} {ri[0]:.2f} {ri[1]:.2f} {ri[2]:.2f} {ari:.2f}')


