# -----------------------------------------------------------
# "Image-Text Alignment using Adaptive Cross-attention with Transformer Encoder for Scene Graphs"
# Juyong Song and Sunghyun Choi
#
# Written by Juyong Song, 2021
# ---------------------------------------------------------------

from transformers import BertTokenizer, AlbertTokenizer, BertModel, AlbertModel, RobertaTokenizer, RobertaModel, DistilBertTokenizer, DistilBertModel

enc_dict = {'bert': {'tokenizer': BertTokenizer,
                     'model': BertModel,
                     'sep_id': 102},
            'albert': {'tokenizer': AlbertTokenizer,
                       'model': AlbertModel,
                       'sep_id': 3},
            'roberta': {'tokenizer': RobertaTokenizer,
                        'model': RobertaModel,
                        'sep_id': 2},
            'distilbert': {'tokenizer': DistilBertTokenizer,
                           'model': DistilBertModel,
                           'sep_id': 102},
            }

model_name_dict = {
    'bert_base': 'bert-base-uncased',  # 108M params
    'bert_large': 'bert-large-uncased',  # 334M params
    'albert_base': 'albert-base-v2',  # 12M params
    'albert_large': 'albert-large-v2',  # 18M params
    'albert_xlarge': 'albert-xlarge-v2',  # 60M params
    'albert_xxlarge': 'albert-xxlarge-v2',  # 235M params
    'roberta_base': 'roberta-base',
    'distilbert_base': 'distilbert-base-uncased',
}


def get_enc_tokenizer(enc_type):
    return enc_dict[enc_type.split('_')[0]]['tokenizer'].from_pretrained(model_name_dict[enc_type])


def get_enc_model(enc_type):
    return enc_dict[enc_type.split('_')[0]]['model'].from_pretrained(model_name_dict[enc_type], return_dict=True)


def get_sep_token_id(enc_type):
    return enc_dict[enc_type.split('_')[0]]['sep_id']
